# Generated by Django 2.1.3 on 2019-03-05 12:30

import django.db.models.deletion
import enumfields.fields
import parler.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="NotificationTemplate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "type",
                    models.CharField(max_length=50, unique=True, verbose_name="type"),
                ),
            ],
            options={
                "verbose_name": "notification",
                "verbose_name_plural": "notifications",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="NotificationTemplateTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                ("subject", models.CharField(max_length=255, verbose_name="subject")),
                (
                    "body_html",
                    models.TextField(blank=True, verbose_name="body, HTML version"),
                ),
                (
                    "body_text",
                    models.TextField(
                        blank=True,
                        help_text="If left blank, the HTML version without HTML tags will be used.",
                        verbose_name="body, plain text version",
                    ),
                ),
                (
                    "master",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="django_ilmoitin.NotificationTemplate",
                    ),
                ),
            ],
            options={
                "verbose_name": "notification Translation",
                "db_table": "django_ilmoitin_notificationtemplate_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
        ),
        migrations.AlterUniqueTogether(
            name="notificationtemplatetranslation",
            unique_together={("language_code", "master")},
        ),
    ]



from django.contrib.admin import site as admin_site
from django.http import HttpResponse
from jinja2 import DebugUndefined, TemplateSyntaxError
from jinja2.sandbox import SandboxedEnvironment
from parler.admin import TranslatableAdmin
from parler.forms import TranslatableModelForm

from .dummy_context import dummy_context
from .models import NotificationTemplate


class NotificationTemplateForm(TranslatableModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Do not allow the admin to choose any of the template types that already
        # exist.
        existing_templates = NotificationTemplate.objects.all()
        if self.instance and self.instance.type:
            existing_templates = existing_templates.exclude(id=self.instance.id)
        used_types = set(existing_templates.values_list("_type", flat=True))
        choices = [x for x in self.fields["_type"].choices if x[0] not in used_types]
        self.fields["_type"].choices = choices


class NotificationTemplateAdmin(TranslatableAdmin):
    form = NotificationTemplateForm
    change_form_template = "admin/preview_template.html"

    def save_model(self, request, obj, form, change):
        if "_preview" not in request.POST:
            super(NotificationTemplateAdmin, self).save_model(
                request, obj, form, change
            )

    def response_change(self, request, obj):
        if "_preview" in request.POST:
            return self.preview(request, obj)
        else:
            return super(NotificationTemplateAdmin, self).response_change(request, obj)

    def response_add(self, request, obj, post_url_continue=None):
        if "_preview" in request.POST:
            return self.preview(request, obj)
        else:
            return super(NotificationTemplateAdmin, self).response_add(
                request, obj, post_url_continue=None
            )

    def preview(self, request, obj, **kwargs):
        env = SandboxedEnvironment(
            trim_blocks=True, lstrip_blocks=True, undefined=DebugUndefined
        )
        try:
            body_html = env.from_string(obj.body_html).render(dummy_context.context)
            return HttpResponse(body_html)
        except TemplateSyntaxError as e:
            return HttpResponse(e)


admin_site.register(NotificationTemplate, NotificationTemplateAdmin)



import logging

from django.db import models
from django.utils.translation import ugettext_lazy as _
from parler.models import TranslatableModel, TranslatedFields

from .registry import notifications

logger = logging.getLogger(__name__)


class NotificationTemplateException(Exception):
    pass


class NotificationTemplate(TranslatableModel):
    _type = models.CharField(
        max_length=50,
        verbose_name=_("type"),
        unique=True,
        db_column="type",
        choices=list(notifications.registry.items()),
    )

    translations = TranslatedFields(
        subject=models.CharField(verbose_name=_("subject"), max_length=255),
        body_html=models.TextField(verbose_name=_("body, HTML version"), blank=True),
        body_text=models.TextField(
            verbose_name=_("body, plain text version"),
            blank=True,
            help_text=_(
                "If left blank, the HTML version without HTML tags will be used."
            ),
        ),
    )

    class Meta:
        verbose_name = _("notification")
        verbose_name_plural = _("notifications")

    def __str__(self):
        return str(self.type)

    @property
    def type(self):
        return notifications.registry.get(self._type)

    @type.setter
    def type(self, value):
        self._type = value



DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sites",
    "django.contrib.admin",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django_ilmoitin",
    "tests",
    "mailer",
)

MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
]

SITE_ID = 1
ROOT_URLCONF = "tests.urls"
DEBUG = True
USE_TZ = True
SECRET_KEY = "sekret"
LANGUAGES = (("fi", "Finnish"), ("en", "English"), ("sv", "Swedish"))



import logging
from collections import namedtuple

from django.conf import settings
from django.core.mail import send_mail as django_send_mail
from django.utils.html import strip_tags
from jinja2 import StrictUndefined
from jinja2.exceptions import TemplateError
from jinja2.sandbox import SandboxedEnvironment
from mailer.engine import send_all
from mailer.models import Message
from parler.utils.context import switch_language

from .models import NotificationTemplate, NotificationTemplateException

logger = logging.getLogger(__name__)

DEFAULT_LANGUAGE = settings.LANGUAGES[0][0]

RenderedTemplate = namedtuple("RenderedTemplate", ("subject", "body_html", "body_text"))


def send_notification(
    email, notification_type, context=None, language=DEFAULT_LANGUAGE
):
    logger.debug(
        'Trying to send notification "{}" to {}.'.format(notification_type, email)
    )

    if context is None:
        context = {}

    try:
        subject, body_html, body_text = render_notification_template(
            notification_type, context, language
        )
    except NotificationTemplate.DoesNotExist:
        logger.debug(
            'NotificationTemplate "{}" does not exist, not sending anything.'.format(
                notification_type
            )
        )
        return
    except NotificationTemplateException as e:
        logger.error(e, exc_info=True)
        return

    if not subject:
        logger.warning(
            'Rendered notification "{}" has an empty subject, not sending anything.'.format(
                notification_type
            )
        )
        return

    send_mail(subject, body_html, body_text, email)

    # also immediately fire django-mailer's commands
    Message.objects.retry_deferred()
    send_all()


def render_notification_template(
    notification_type, context, language_code=DEFAULT_LANGUAGE
):
    """
    Render a notification template with given context in given language

    Returns a namedtuple containing all content fields (subject, body_html, body_text) of the template.
    """
    template = NotificationTemplate.objects.get(_type=notification_type)
    env = SandboxedEnvironment(
        trim_blocks=True, lstrip_blocks=True, undefined=StrictUndefined
    )

    with switch_language(template, language_code):
        try:
            subject = env.from_string(template.subject).render(context)
            body_html = env.from_string(template.body_html).render(context)

            if template.body_text:
                body_text = env.from_string(template.body_text).render(context)
            else:
                body_text = strip_tags(body_html)

            return RenderedTemplate(subject, body_html, body_text)

        except TemplateError as e:
            raise NotificationTemplateException(e) from e


def send_mail(subject, body_html, body_text, to_address):
    logger.info('Sending notification email to {}: "{}"'.format(to_address, subject))
    django_send_mail(
        subject,
        body_text,
        settings.DEFAULT_FROM_EMAIL,
        [to_address],
        html_message=body_html,
    )



import pytest
from django.conf import settings
from django.core import mail

from django_ilmoitin.models import NotificationTemplate, NotificationTemplateException
from django_ilmoitin.registry import notifications
from django_ilmoitin.utils import render_notification_template, send_notification


@pytest.fixture
def notification_template(settings):
    settings.LANGUAGES = (("fi", "Finnish"), ("en", "English"))
    notifications.register("event_created", "Event created")
    notifications.register("event_approved", "Event approved")
    template = NotificationTemplate.objects.language("en").create(
        type="event_created",
        subject="test subject, variable value: {{ subject_var }}!",
        body_html="<b>test body HTML</b>, variable value: {{ body_html_var }}!",
        body_text="test body text, variable value: {{ body_text_var }}!",
    )
    template.set_current_language("fi")
    template.subject = "testiotsikko, muuttujan arvo: {{ subject_var }}!"
    template.body_html = (
        "<b>testihötömölöruumis</b>, muuttujan arvo: {{ body_html_var }}!"
    )
    template.body_text = "testitekstiruumis, muuttujan arvo: {{ body_text_var }}!"

    template.save()

    return template


@pytest.mark.django_db
def test_notification_template_rendering(notification_template):
    context = {
        "extra_var": "foo",
        "subject_var": "bar",
        "body_html_var": "html_baz",
        "body_text_var": "text_baz",
    }

    rendered = render_notification_template("event_created", context, "en")
    assert len(rendered) == 3
    assert rendered.subject == "test subject, variable value: bar!"
    assert rendered.body_html == "<b>test body HTML</b>, variable value: html_baz!"
    assert rendered.body_text == "test body text, variable value: text_baz!"

    rendered = render_notification_template("event_created", context, "fi")
    assert len(rendered) == 3
    assert rendered.subject == "testiotsikko, muuttujan arvo: bar!"
    assert rendered.body_html == "<b>testihötömölöruumis</b>, muuttujan arvo: html_baz!"
    assert rendered.body_text == "testitekstiruumis, muuttujan arvo: text_baz!"


@pytest.mark.django_db
def test_notification_template_rendering_no_body_text_provided(notification_template):
    context = {
        "extra_var": "foo",
        "subject_var": "bar",
        "body_html_var": "html_baz",
        "body_text_var": "text_baz",
    }
    notification_template.set_current_language("fi")
    notification_template.body_text = ""
    notification_template.save()
    notification_template.set_current_language("en")
    notification_template.body_text = ""
    notification_template.save()

    rendered = render_notification_template("event_created", context, "en")
    assert len(rendered) == 3
    assert rendered.subject == "test subject, variable value: bar!"
    assert rendered.body_html == "<b>test body HTML</b>, variable value: html_baz!"
    assert rendered.body_text == "test body HTML, variable value: html_baz!"

    rendered = render_notification_template("event_created", context, "fi")
    assert len(rendered) == 3
    assert rendered.subject == "testiotsikko, muuttujan arvo: bar!"
    assert rendered.body_html == "<b>testihötömölöruumis</b>, muuttujan arvo: html_baz!"
    assert rendered.body_text == "testihötömölöruumis, muuttujan arvo: html_baz!"


@pytest.mark.django_db
def test_undefined_rendering_context_variable(notification_template):
    context = {"extra_var": "foo", "subject_var": "bar", "body_text_var": "baz"}

    with pytest.raises(NotificationTemplateException) as e:
        render_notification_template("event_created", context, "fi")
    assert "'body_html_var' is undefined" in str(e)


@pytest.mark.django_db
def test_notification_sending(notification_template):
    context = {
        "extra_var": "foo",
        "subject_var": "bar",
        "body_html_var": "html_baz",
        "body_text_var": "text_baz",
    }
    send_notification("foo@bar.fi", "event_created", context, "fi")

    assert len(mail.outbox) == 1
    message = mail.outbox[0]
    subject = message.subject
    body_html = next(a[0] for a in message.alternatives if a[1] == "text/html")
    body_text = message.body

    assert message.to == ["foo@bar.fi"]
    assert message.from_email == settings.DEFAULT_FROM_EMAIL
    assert subject == "testiotsikko, muuttujan arvo: bar!"
    assert body_html == "<b>testihötömölöruumis</b>, muuttujan arvo: html_baz!"
    assert body_text == "testitekstiruumis, muuttujan arvo: text_baz!"
